@@ -1,5 +1,5 @@
-# $Date: 2007-07-05 07:31:01 -0700 (Thu, 05 Jul 2007) $
-# $Revision: 125 $
+# $Date: 2007-07-05 14:48:16 -0700 (Thu, 05 Jul 2007) $
+# $Revision: 128 $
 # $Author: david.romano $
 # ex: set ts=8 sw=4 et
 #########################################################################
@@ -9,7 +9,7 @@ use warnings;
 use strict;
 use Carp;
 
-use version; our $VERSION = qv('0.3.7');
+use version; our $VERSION = qv('0.3.8');
 
 sub base { return shift->{'base'}; }
 
@@ -24,7 +24,15 @@ sub new {
     return $self;
 }
 
-sub query { return shift->base->call( 'fql.query', @_ ) }
+# always return an array reference. The value returned by Facebook is a hash
+# reference when the are no results, so this shouldn't be a problem.
+sub query {
+    my $self = shift;
+    my $response = $self->base->call( 'fql.query', @_ );
+
+    return $response if !$self->base->parse && !$self->base->format eq 'JSON';
+    return ref $response eq 'HASH' ? [] : $response;
+}
 
 1;    # Magic true value required at end of module
 __END__
@@ -35,7 +43,7 @@ WWW::Facebook::API::FQL - Facebook Query Language
 
 =head1 VERSION
 
-This document describes WWW::Facebook::API::FQL version 0.3.7
+This document describes WWW::Facebook::API::FQL version 0.3.8
 
 =head1 SYNOPSIS
 
@@ -69,6 +77,11 @@ The fql.query method of the Facebook API:
 
     $response = $client->fql->query( query => 'FQL query' );
 
+When C<$self->base->parse> returns true, and when the format is JSON, the
+response will always be an array reference. (Facebook actually returns an
+empty hash reference when empty, but an array reference otherwise. This is
+meant to make developer more consistent.)
+
 =back
 
 =head1 DIAGNOSTICS
